"use strict";
/*
Copyright 2016 Balena

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const errors = tslib_1.__importStar(require("balena-errors"));
const once = require("lodash/once");
const util_1 = require("../util");
const types_1 = require("../util/types");
const getReleaseModel = function (deps, opts) {
    const { pine, 
    // Do not destructure sub-modules, to allow lazy loading only when needed.
    sdkInstance, } = deps;
    const { buildDependentResource } = 
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    require('../util/dependent-resource');
    const builderHelper = once(() => {
        const { BuilderHelper } = 
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        require('../util/builder');
        return new BuilderHelper(deps, opts);
    });
    const tagsModel = buildDependentResource({ pine }, {
        resourceName: 'release_tag',
        resourceKeyField: 'tag_key',
        parentResourceName: 'release',
        getResourceId(commitOrIdOrRawVersion) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { id } = yield get(commitOrIdOrRawVersion, { $select: 'id' });
                return id;
            });
        },
    });
    /**
     * @summary Get a specific release
     * @name get
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
     * @param {Object} [options={}] - extra pine options to use
     * @fulfil {Object} - release
     * @returns {Promise}
     *
     * @example
     * balena.models.release.get(123).then(function(release) {
     * 	console.log(release);
     * });
     *
     * @example
     * balena.models.release.get('7cf02a6').then(function(release) {
     * 	console.log(release);
     * });
     *
     * @example
     * balena.models.release.get({application: 456, raw_version: '0.0.0'}).then(function(release) {
     * 	console.log(release);
     * });
     */
    function get(commitOrIdOrRawVersion_1) {
        return tslib_1.__awaiter(this, arguments, void 0, function* (commitOrIdOrRawVersion, options = {}) {
            if (commitOrIdOrRawVersion == null) {
                throw new errors.BalenaReleaseNotFound(commitOrIdOrRawVersion);
            }
            if ((0, util_1.isId)(commitOrIdOrRawVersion)) {
                const release = yield pine.get({
                    resource: 'release',
                    id: commitOrIdOrRawVersion,
                    options: (0, util_1.mergePineOptions)({}, options),
                });
                if (release == null) {
                    throw new errors.BalenaReleaseNotFound(commitOrIdOrRawVersion);
                }
                return release;
            }
            else {
                let $filter;
                if (typeof commitOrIdOrRawVersion === 'object') {
                    const { rawVersion, application } = commitOrIdOrRawVersion;
                    const { id } = yield sdkInstance.models.application.get(application, {
                        $select: 'id',
                    });
                    $filter = {
                        raw_version: rawVersion,
                        belongs_to__application: id,
                    };
                }
                else {
                    $filter = {
                        commit: { $startswith: commitOrIdOrRawVersion },
                    };
                }
                const releases = yield pine.get({
                    resource: 'release',
                    options: (0, util_1.mergePineOptions)({
                        $filter,
                    }, options),
                });
                if (releases.length === 0) {
                    throw new errors.BalenaReleaseNotFound(typeof commitOrIdOrRawVersion === 'string'
                        ? commitOrIdOrRawVersion
                        : `unique pair ${Object.keys(commitOrIdOrRawVersion).join(' & ')}: ${Object.values(commitOrIdOrRawVersion).join(' & ')}`);
                }
                if (releases.length > 1) {
                    throw new errors.BalenaAmbiguousRelease(commitOrIdOrRawVersion);
                }
                return releases[0];
            }
        });
    }
    /**
     * @summary Get a specific release with the details of the images built
     * @name getWithImageDetails
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @description
     * This method does not map exactly to the underlying model: it runs a
     * larger prebuilt query, and reformats it into an easy to use and
     * understand format. If you want significantly more control, or to see the
     * raw model directly, use `release.get(id, options)` instead.
     *
     * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
     * @param {Object} [options={}] - a map of extra pine options
     * @param {Boolean} [options.release={}] - extra pine options for releases
     * @param {Object} [options.image={}] - extra pine options for images
     * @fulfil {Object} - release with image details
     * @returns {Promise}
     *
     * @example
     * balena.models.release.getWithImageDetails(123).then(function(release) {
     * 	console.log(release);
     * });
     *
     * @example
     * balena.models.release.getWithImageDetails('7cf02a6').then(function(release) {
     * 	console.log(release);
     * });
     *
     * @example
     * balena.models.release.getWithImageDetails({application: 456, raw_version: '0.0.0'}).then(function(release) {
     * 	console.log(release);
     * });
     *
     * @example
     * balena.models.release.getWithImageDetails(123, { image: { $select: 'build_log' } })
     * .then(function(release) {
     * 	console.log(release.images[0].build_log);
     * });
     */
    function getWithImageDetails(commitOrIdOrRawVersion_1) {
        return tslib_1.__awaiter(this, arguments, void 0, function* (commitOrIdOrRawVersion, options = {}) {
            const baseImageOptions = {
                $select: 'id',
                $expand: {
                    is_a_build_of__service: {
                        $select: 'service_name',
                    },
                },
            };
            const baseReleaseOptions = {
                $expand: {
                    release_image: {
                        $expand: {
                            image: (0, util_1.mergePineOptions)(baseImageOptions, options.image),
                        },
                    },
                    is_created_by__user: {
                        $select: (0, types_1.toWritable)(['id', 'username']),
                    },
                },
            };
            const rawRelease = (yield get(commitOrIdOrRawVersion, (0, util_1.mergePineOptions)(baseReleaseOptions, options.release)));
            const release = rawRelease;
            // Squash .release_image[x].image[0] into a simple array
            const images = rawRelease.release_image.map((imageJoin) => imageJoin.image[0]);
            delete release.release_image;
            release.images = images
                .map(function (_a) {
                var { is_a_build_of__service } = _a, imageData = tslib_1.__rest(_a, ["is_a_build_of__service"]);
                const image = Object.assign(Object.assign({}, imageData), { service_name: is_a_build_of__service[0].service_name });
                return image;
            })
                .sort((a, b) => a.service_name.localeCompare(b.service_name));
            release.user = rawRelease.is_created_by__user[0];
            return release;
        });
    }
    /**
     * @summary Get all releases from an application
     * @name getAllByApplication
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number} slugOrUuidOrId - application slug (string), uuid (string) or id (number)
     * @param {Object} [options={}] - extra pine options to use
     * @fulfil {Object[]} - releases
     * @returns {Promise}
     *
     * @example
     * balena.models.release.getAllByApplication('myorganization/myapp').then(function(releases) {
     * 	console.log(releases);
     * });
     *
     * @example
     * balena.models.release.getAllByApplication(123).then(function(releases) {
     * 	console.log(releases);
     * });
     */
    function getAllByApplication(slugOrUuidOrId_1) {
        return tslib_1.__awaiter(this, arguments, void 0, function* (slugOrUuidOrId, options = {}) {
            const { id } = yield sdkInstance.models.application.get(slugOrUuidOrId, {
                $select: 'id',
            });
            return yield pine.get({
                resource: 'release',
                options: (0, util_1.mergePineOptions)({
                    $filter: {
                        belongs_to__application: id,
                    },
                    $orderby: 'created_at desc',
                }, options),
            });
        });
    }
    /**
     * @summary Get the latest successful release for an application
     * @name getLatestByApplication
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number} slugOrUuidOrId - application slug (string), uuid (string) or id (number)
     * @param {Object} [options={}] - extra pine options to use
     * @fulfil {Object|undefined} - release
     * @returns {Promise}
     *
     * @example
     * balena.models.release.getLatestByApplication('myorganization/myapp').then(function(releases) {
     * 	console.log(releases);
     * });
     *
     * @example
     * balena.models.release.getLatestByApplication(123).then(function(releases) {
     * 	console.log(releases);
     * });
     */
    function getLatestByApplication(slugOrUuidOrId_1) {
        return tslib_1.__awaiter(this, arguments, void 0, function* (slugOrUuidOrId, options = {}) {
            const [release] = yield getAllByApplication(slugOrUuidOrId, (0, util_1.mergePineOptions)({
                $top: 1,
                $filter: {
                    status: 'success',
                },
            }, options));
            return release;
        });
    }
    /**
     * @summary Create a new release built from the source in the provided url
     * @name createFromUrl
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number} slugOrUuidOrId - application slug (string), uuid (string) or id (number)
     * @param {Object} urlDeployOptions - builder options
     * @param {String} urlDeployOptions.url - a url with a tarball of the project to build
     * @param {Boolean} [urlDeployOptions.shouldFlatten=true] - Should be true when the tarball includes an extra root folder with all the content
     * @fulfil {number} - release ID
     * @returns {Promise}
     *
     * @example
     * balena.models.release.createFromUrl('myorganization/myapp', { url: 'https://github.com/balena-io-projects/simple-server-node/archive/v1.0.0.tar.gz' }).then(function(releaseId) {
     * 	console.log(releaseId);
     * });
     *
     * @example
     * balena.models.release.createFromUrl(123, { url: 'https://github.com/balena-io-projects/simple-server-node/archive/v1.0.0.tar.gz' }).then(function(releaseId) {
     * 	console.log(releaseId);
     * });
     */
    function createFromUrl(slugOrUuidOrId, urlDeployOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const appOptions = {
                $select: 'app_name',
                $expand: {
                    organization: {
                        $select: 'handle',
                    },
                },
            };
            const { app_name, organization } = (yield sdkInstance.models.application.get(slugOrUuidOrId, appOptions));
            return yield builderHelper().buildFromUrl(organization[0].handle, app_name, urlDeployOptions);
        });
    }
    /**
     * @summary Finalizes a draft release
     * @name finalize
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
     * @fulfil {void}
     * @returns {Promise}
     *
     * @example
     * balena.models.release.finalize(123).then(function() {
     * 	console.log('finalized!');
     * });
     *
     * @example
     * balena.models.release.finalize('7cf02a6').then(function() {
     * 	console.log('finalized!');
     * });
     *
     * @example
     * balena.models.release.finalize({application: 456, raw_version: '0.0.0'}).then(function(release) {
     * 	console.log('finalized!');
     * });
     *
     */
    function finalize(commitOrIdOrRawVersion) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { id } = yield get(commitOrIdOrRawVersion, { $select: 'id' });
            yield pine.patch({
                resource: 'release',
                id,
                body: {
                    is_final: true,
                },
            });
        });
    }
    /**
     * @summary Set the is_invalidated property of a release to true or false
     * @name setIsInvalidated
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
     * @param {Boolean} isInvalidated - boolean value, true for invalidated, false for validated
     * @fulfil {void}
     * @returns {Promise}
     *
     * @example
     * balena.models.release.setIsInvalidated(123, true).then(function() {
     * 	console.log('invalidated!');
     * });
     *
     * @example
     * balena.models.release.setIsInvalidated('7cf02a6', true).then(function() {
     * 	console.log('invalidated!');
     * });
     *
     * @example
     * balena.models.release.setIsInvalidated({application: 456, raw_version: '0.0.0'}).then(function(release) {
     * 	console.log('invalidated!);
     * });
     *
     * @example
     * balena.models.release.setIsInvalidated(123, false).then(function() {
     * 	console.log('validated!');
     * });
     *
     * @example
     * balena.models.release.setIsInvalidated('7cf02a6', false).then(function() {
     * 	console.log('validated!');
     * });
     *
     */
    function setIsInvalidated(commitOrIdOrRawVersion, isInvalidated) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { id } = yield get(commitOrIdOrRawVersion, { $select: 'id' });
            yield pine.patch({
                resource: 'release',
                id,
                body: {
                    is_invalidated: isInvalidated,
                },
            });
        });
    }
    /**
     * @summary Add a note to a release
     * @name setNote
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
     * @param {String|Null} noteOrNull - the note
     *
     * @returns {Promise}
     *
     * @example
     * balena.models.release.setNote('7cf02a6', 'My useful note');
     *
     * @example
     * balena.models.release.setNote(123, 'My useful note');
     *
     * @example
     * balena.models.release.setNote({ application: 456, rawVersion: '0.0.0' }, 'My useful note');
     *
     */
    function setNote(commitOrIdOrRawVersion, noteOrNull) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { id } = yield get(commitOrIdOrRawVersion, { $select: 'id' });
            yield pine.patch({
                resource: 'release',
                id,
                body: {
                    note: noteOrNull,
                },
            });
        });
    }
    /**
     * @summary Add a known issue list to a release
     * @name setKnownIssueList
     * @public
     * @function
     * @memberof balena.models.release
     *
     * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
     * @param {String|Null} knownIssueListOrNull - the known issue list
     *
     * @returns {Promise}
     *
     * @example
     * balena.models.release.setKnownIssueList('7cf02a6', 'This is an issue');
     *
     * @example
     * balena.models.release.setKnownIssueList(123, 'This is an issue');
     *
     * @example
     * balena.models.release.setKnownIssueList({application: 456, rawVersion: '0.0.0'}, 'This is an issue');
     */
    function setKnownIssueList(commitOrIdOrRawVersion, knownIssueListOrNull) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { id } = yield get(commitOrIdOrRawVersion, { $select: 'id' });
            yield pine.patch({
                resource: 'release',
                id,
                body: {
                    known_issue_list: knownIssueListOrNull,
                },
            });
        });
    }
    /**
     * @namespace balena.models.release.tags
     * @memberof balena.models.release
     */
    const tags = {
        /**
         * @summary Get all release tags for an application
         * @name getAllByApplication
         * @public
         * @function
         * @memberof balena.models.release.tags
         *
         * @param {String|Number} slugOrUuidOrId - application slug (string), uuid (string) or id (number)
         * @param {Object} [options={}] - extra pine options to use
         * @fulfil {Object[]} - release tags
         * @returns {Promise}
         *
         * @example
         * balena.models.release.tags.getAllByApplication('myorganization/myapp').then(function(tags) {
         * 	console.log(tags);
         * });
         *
         * @example
         * balena.models.release.tags.getAllByApplication(999999).then(function(tags) {
         * 	console.log(tags);
         * });
         */
        getAllByApplication(slugOrUuidOrId_1) {
            return tslib_1.__awaiter(this, arguments, void 0, function* (slugOrUuidOrId, options = {}) {
                const { id } = yield sdkInstance.models.application.get(slugOrUuidOrId, {
                    $select: 'id',
                });
                return yield tagsModel.getAll((0, util_1.mergePineOptions)({
                    $filter: {
                        release: {
                            $any: {
                                $alias: 'r',
                                $expr: {
                                    r: {
                                        belongs_to__application: id,
                                    },
                                },
                            },
                        },
                    },
                }, options));
            });
        },
        /**
         * @summary Get all release tags for a release
         * @name getAllByRelease
         * @public
         * @function
         * @memberof balena.models.release.tags
         *
         * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
         * @param {Object} [options={}] - extra pine options to use
         * @fulfil {Object[]} - release tags
         * @returns {Promise}
         *
         * @example
         * balena.models.release.tags.getAllByRelease(123).then(function(tags) {
         * 	console.log(tags);
         * });
         *
         * @example
         * balena.models.release.tags.getAllByRelease('7cf02a6').then(function(tags) {
         * 	console.log(tags);
         * });
         *
         * @example
         * balena.models.release.tags.getAllByRelease({application: 456, rawVersion: '0.0.0'}).then(function(tags) {
         * 	console.log(tags);
         * });
         */
        getAllByRelease(commitOrIdOrRawVersion_1) {
            return tslib_1.__awaiter(this, arguments, void 0, function* (commitOrIdOrRawVersion, options = {}) {
                const releaseOpts = {
                    $select: 'id',
                    $expand: {
                        release_tag: (0, util_1.mergePineOptions)({ $orderby: 'tag_key asc' }, options),
                    },
                };
                const release = (yield get(commitOrIdOrRawVersion, releaseOpts));
                return release.release_tag;
            });
        },
        /**
         * @summary Set a release tag
         * @name set
         * @public
         * @function
         * @memberof balena.models.release.tags
         *
         * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
         * @param {String} tagKey - tag key
         * @param {String|undefined} value - tag value
         *
         * @returns {Promise}
         *
         * @example
         * balena.models.release.tags.set(123, 'EDITOR', 'vim');
         *
         * @example
         * balena.models.release.tags.set('7cf02a6', 'EDITOR', 'vim');
         *
         * @example
         * balena.models.release.tags.set({application: 456, rawVersion: '0.0.0'}, 'EDITOR', 'vim');
         */
        set: tagsModel.set,
        /**
         * @summary Remove a release tag
         * @name remove
         * @public
         * @function
         * @memberof balena.models.release.tags
         *
         * @param {String|Number|Object} commitOrIdOrRawVersion - release commit (string) or id (number) or an object with the unique `application` (number or string) & `rawVersion` (string) pair of the release
         * @param {String} tagKey - tag key
         * @returns {Promise}
         *
         * @example
         * balena.models.release.tags.remove(123, 'EDITOR');
         *
         * @example
         * balena.models.release.tags.remove('7cf02a6', 'EDITOR');
         *
         * @example
         * balena.models.release.tags.remove({application: 456, rawVersion: '0.0.0'}, 'EDITOR');
         */
        remove: tagsModel.remove,
    };
    return {
        get,
        getAllByApplication,
        getLatestByApplication,
        getWithImageDetails,
        createFromUrl,
        finalize,
        setIsInvalidated,
        setNote,
        setKnownIssueList,
        tags,
    };
};
exports.default = getReleaseModel;
